﻿using JSKExcelWrapper;
using NetOffice.ExcelApi;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace JSKExcelWrapperSample
{
    /// <summary>
    /// JSKExcelWrapper動作確認用アプリ
    /// ※必ずこのアプリでExcel出力の動作確認を行ってから、製品をご購入下さい
    /// 購入先　https://coconala.com/services/1408367
    /// 2020/09/22 Juraku-Software https://juraku-software.net/
    /// </summary>
    public partial class FormMain : Form
    {
        #region コンストラクタ/フォームロード
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FormMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォームロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormMain_Load(object sender, EventArgs e)
        {
            //特に必要な処理はありません
        }
        #endregion

        /// <summary>
        /// ①基本機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonBase_Click(object sender, EventArgs e)
        {
            //出力中は操作できないように画面を操作不可にする
            ShowWaitMsg();

            //Excel起動
            using (var excelWrapper = new JSKExcelWrapper.JSKExcelWrapper())
            {
                //◆シート取得
                var worksheet = excelWrapper.GetWorksheet("Sheet1");


                //◆文字入力
                excelWrapper.SetCellValue(worksheet, "A1", "A1へ書き込み");
                excelWrapper.SetCellValue(worksheet, "A2", Color.Red, "A2へ書き込み(文字列赤）");        //文字色変更
                excelWrapper.SetCellValue(worksheet, "A3:B4", Color.Green, "複数セルまとめて記載");

                excelWrapper.SetCellValue(worksheet, "C1", "1");
                excelWrapper.SetCellValue(worksheet, "C2", "2");
                excelWrapper.SetCellValue(worksheet, "C3", "=C1+C2");       //計算式入力

                //◆列幅変更（A列を変更）
                excelWrapper.SetColumnWidth(worksheet, "A", 20);

                //◆行高さ変更（1行目～3行目を変更）
                excelWrapper.SetRowHeight(worksheet, "1:4", 20);

                //◆罫線
                excelWrapper.SetLine(worksheet, "C1:C3");

                //◆セル背景色
                excelWrapper.SetCellBackColor(worksheet, "C3", Color.Green);

                //◆文字取得
                Console.WriteLine("取得した文字：" + excelWrapper.GetCellValue(worksheet, "A1"));
                Console.WriteLine("取得した文字：" + excelWrapper.GetCellValue(worksheet, "A2:A3"));    //複数セル纏めて
                Console.WriteLine("取得した文字：" + excelWrapper.GetCellValue(worksheet, "C3"));       //検索結果の値が返る

                //◆日付形式で表示
                excelWrapper.SetCellValue(worksheet, "D1", "44090");
                excelWrapper.SetCellFormat(worksheet, "D1", "yyyy/MM/dd");

                //◆数値の3桁区切りで表示
                excelWrapper.SetCellValue(worksheet, "D2:D3", "123456789");
                excelWrapper.SetCellFormat(worksheet, "D2:D3", "#,##0");


                //出力先パス（このEXEと同じフォルダに出力）
                var fileBase = Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath) + @"\サンプル出力①_" + DateTime.Now.ToString("yyyyMMddHHmmss");

                //◆ファイル出力
                excelWrapper.SaveExcel(fileBase + ".xlsx");     //Excelファイル出力
                excelWrapper.SavePDF(fileBase + ".pdf");        //PDFファイル出力

                //◆プレビュー表示
                excelWrapper.Preview();
            }

            //出力が完了したので、画面を操作可能にする
            HideWaitMsg();
        }

        /// <summary>
        /// ②シェイプ図形、画像
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShape_Click(object sender, EventArgs e)
        {
            //出力中は操作できないように画面を操作不可にする
            ShowWaitMsg();

            //Excel起動
            using (var excelWrapper = new JSKExcelWrapper.JSKExcelWrapper())
            {
                //◆シート取得
                var worksheet = excelWrapper.GetWorksheet("Sheet1");

                //◆四角形追加
                excelWrapper.AddShapeSquare(worksheet, "A1", 100, 100, Color.Red);          //表示位置をセル番地を指定する場合
                excelWrapper.AddShapeSquare(worksheet, 120, 0, 100, 100, Color.Blue);     //表示位置を座標で指定する場合

                //◆円形追加
                excelWrapper.AddShapeCircle(worksheet, "A8", 100, 100, Color.Red);          //表示位置をセル番地を指定する場合
                excelWrapper.AddShapeCircle(worksheet, 120, 140, 100, 100, Color.Blue);     //表示位置を座標で指定する場合

                //◆画像ファイル追加

                //画像ファイルパス取得
                var picturePath = Path.Combine(Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath), @"テンプレート\画像サンプル（いらすとやさん）.png");

                excelWrapper.AddPicture(worksheet, picturePath, "A20", 100, 100);           //表示位置をセル番地を指定する場合
                excelWrapper.AddPicture(worksheet, picturePath, 100, 350, 100, 100);        //表示位置を座標で指定する場合


                //出力先パス（このEXEと同じフォルダに出力）
                var fileBase = Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath) + @"\サンプル出力②_" + DateTime.Now.ToString("yyyyMMddHHmmss");

                //◆ファイル出力
                excelWrapper.SaveExcel(fileBase + ".xlsx");     //Excelファイル出力
                excelWrapper.SavePDF(fileBase + ".pdf");        //PDFファイル出力

                //◆プレビュー表示
                excelWrapper.Preview();
            }

            //出力が完了したので、画面を操作可能にする
            HideWaitMsg();
        }

        /// <summary>
        /// ③グラフ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonChart_Click(object sender, EventArgs e)
        {
            //出力中は操作できないように画面を操作不可にする
            ShowWaitMsg();

            //Excel起動
            using (var excelWrapper = new JSKExcelWrapper.JSKExcelWrapper())
            {
                //◆シート取得
                var worksheet = excelWrapper.GetWorksheet("Sheet1");

                //◆グラフ用のデータを記載
                //第2引数で複数のセル範囲を指定した場合、最後の引数にそれぞれのセルに記載する値を指定できる。
                //その場合、引数に指定された値が左から１行ずつ記載されていく（A1→B1→A2→B2→A3→B3→A4→B4の順）
                excelWrapper.SetCellValue(worksheet, "A1:B4",
                    "項目名", "値",
                    "項目1", "50",
                    "項目2", "75",
                    "項目3", "60");

                //◆枠線を追加
                excelWrapper.SetLine(worksheet, "A1:B4");

                //◆縦棒グラフ
                excelWrapper.AddChartVerticalBar(worksheet, "A1:B4", "縦棒グラフ（セル指定）", "A6", 200, 200);    //表示位置をセル番地を指定する場合
                excelWrapper.AddChartVerticalBar(worksheet, "A1:B4", "縦棒グラフ（座標指定）", 220, 90, 200, 200);  //表示位置を座標で指定する場合

                //◆折れ線グラフ
                excelWrapper.AddChartLine(worksheet, "A1:B4", "折れ線グラフ（セル指定）", "A17", 200, 200);         //表示位置をセル番地を指定する場合
                excelWrapper.AddChartLine(worksheet, "A1:B4", "折れ線グラフ（座標指定）", 220, 300, 200, 200);      //表示位置を座標で指定する場合

                //◆円グラフ
                excelWrapper.AddChartCircle(worksheet, "A1:B4", "円グラフ（セル指定）", "A28", 200, 200);         //表示位置をセル番地を指定する場合
                excelWrapper.AddChartCircle(worksheet, "A1:B4", "円グラフ（座標指定）", 220, 510, 200, 200);      //表示位置を座標で指定する場合


                //出力先パス（このEXEと同じフォルダに出力）
                var fileBase = Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath) + @"\サンプル出力③_" + DateTime.Now.ToString("yyyyMMddHHmmss");

                //◆ファイル出力
                excelWrapper.SaveExcel(fileBase + ".xlsx");     //Excelファイル出力
                excelWrapper.SavePDF(fileBase + ".pdf");        //PDFファイル出力

                //◆プレビュー表示
                excelWrapper.Preview();
            }

            //出力が完了したので、画面を操作可能にする
            HideWaitMsg();
        }

        /// <summary>
        /// ④シート操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSheet_Click(object sender, EventArgs e)
        {
            //出力中は操作できないように画面を操作不可にする
            ShowWaitMsg();

            //Excel起動
            using (var excelWrapper = new JSKExcelWrapper.JSKExcelWrapper())
            {
                //◆シート追加
                var worksheetSample1 = excelWrapper.AddWorksheet("Sample1");
                var worksheetSample2 = excelWrapper.AddWorksheet("Sample2");

                //◆シート名変更（__Sample__から、NewSheetに変更）
                worksheetSample2 = excelWrapper.ChangeWorksheetName("Sample2", "NewSheet2");

                //◆シート削除（Sheet1というシートが自動作成されるので削除する）
                excelWrapper.DeleteWorksheet("Sheet1");

                //◆シート非表示
                excelWrapper.HideWorksheet(worksheetSample2);

                //◆シート表示
                excelWrapper.ShowWorksheet(worksheetSample2);


                //◆文字入力（A1～D100の全セルに「テスト」という文字を書き込む）
                excelWrapper.SetCellValue(worksheetSample1, "A1:H30", "テスト");

                //◆罫線
                excelWrapper.SetLine(worksheetSample1, "A1:H30");

                //◆改ページクリア（現在設定されている改ページをすべて消す）
                excelWrapper.ClearAlllBreaks(worksheetSample1);


                //◆改ページ設定（縦方向）
                excelWrapper.SetVPageBreaks(worksheetSample1, "E15");

                //◆改ページ設定（横方向）
                excelWrapper.SetHPageBreaks(worksheetSample1, "A16");

                //◆シートコピー（コピー元、コピー先）
                excelWrapper.CopyWorksheet("Sample1", "コピーしたシート");


                //出力先パス（このEXEと同じフォルダに出力）
                var fileBase = Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath) + @"\サンプル出力④_" + DateTime.Now.ToString("yyyyMMddHHmmss");

                //◆ファイル出力
                excelWrapper.SaveExcel(fileBase + ".xlsx");     //Excelファイル出力
                excelWrapper.SavePDF(fileBase + ".pdf");        //PDFファイル出力

                //◆プレビュー表示
                excelWrapper.Preview();
            }

            //出力が完了したので、画面を操作可能にする
            HideWaitMsg();
        }

        /// <summary>
        /// ⑤帳票形式でのファイル出力
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonReport_Click(object sender, EventArgs e)
        {
            //出力中は操作できないように画面を操作不可にする
            ShowWaitMsg();

            //Excel起動
            using (var excelWrapper = new JSKExcelWrapper.JSKExcelWrapper())
            {
                //テンプレート（ひな形）ファイルパス取得
                var templatePath = Path.Combine(Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath), @"テンプレート\帳票テンプレート.xlsx");

                //◆Excelファイル開く
                excelWrapper.OpenWorkbook(templatePath);

                //◆シート取得
                Worksheet worksheet = excelWrapper.GetWorksheet("template");

                //帳票形式で表示させるためのダミーデータ
                var dummyData = new List<string[]>()
                {
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                    new string[]{"A001", "部品1", "TEST-A001", "汎用部品1", "メモ1"},
                    new string[]{"A002", "部品2", "TEST-A002", "汎用部品2", "メモ2"},
                    new string[]{"A003", "部品3", "TEST-A003", "汎用部品3", "メモ3"},
                    new string[]{"A004", "部品4", "TEST-A004", "汎用部品4", "メモ4"},
                    new string[]{"A005", "部品5", "TEST-A005", "汎用部品5", "メモ5"},
                    new string[]{"A006", "部品6", "TEST-A006", "汎用部品6", "メモ6"},
                };

                //◆テンプレートファイルの各セルに設定されたセル名から、転記すするセル番地を取得する
                var indexCode1 = excelWrapper.GetExcelIndexData(worksheet, "コード");
                var indexName1 = excelWrapper.GetExcelIndexData(worksheet, "名称");
                var indexCode2 = excelWrapper.GetExcelIndexData(worksheet, "コード2");
                var indexName2 = excelWrapper.GetExcelIndexData(worksheet, "名称2");
                var indexMemo = excelWrapper.GetExcelIndexData(worksheet, "メモ");

                //◆データ数分、明細行を行コピーする（罫線や書式もコピーされる）
                excelWrapper.CopyRow(worksheet,
                    indexCode1.rowIndex/*コピー行の行番号（開始）*/,
                    indexCode1.rowIndex/*コピー行の行番号（終了）*/,
                    indexCode1.rowIndex + 1/*貼り付ける位置の行番号（開始）  最初の1行はすでに存在しているので、次の行からコピーする*/,
                    indexCode1.rowIndex + dummyData.Count - 1/*貼り付ける位置の行番号（終了）  最初の1行はすでに存在しているので、-1する*/);

                //データの行数分回す
                foreach (var data in dummyData)
                {
                    //◆転記
                    excelWrapper.SetCellValue(worksheet, indexCode1, data[0]);
                    excelWrapper.SetCellValue(worksheet, indexName1, data[1]);
                    excelWrapper.SetCellValue(worksheet, indexCode2, data[2]);
                    excelWrapper.SetCellValue(worksheet, indexName2, data[3]);
                    excelWrapper.SetCellValue(worksheet, indexMemo, data[4]);

                    //◆記載が終わったので、次の行へインデックスをずらす
                    indexCode1.NextRow();
                    indexName1.NextRow();
                    indexCode2.NextRow();
                    indexName2.NextRow();
                    indexMemo.NextRow();
                }


                //出力先パス（このEXEと同じフォルダに出力）
                var fileBase = Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath) + @"\サンプル出力⑤_" + DateTime.Now.ToString("yyyyMMddHHmmss");

                //◆ファイル出力
                excelWrapper.SaveExcel(fileBase + ".xlsx");     //Excelファイル出力
                excelWrapper.SavePDF(fileBase + ".pdf");        //PDFファイル出力

                //◆プレビュー表示
                excelWrapper.Preview();
            }

            //出力が完了したので、画面を操作可能にする
            HideWaitMsg();
        }







        /// <summary>
        /// Excel出力中である旨のメッセージを表示する
        /// </summary>
        private void ShowWaitMsg()
        {
            //このフォームを操作不可にする
            this.Enabled = false;

            //画面全体に処理中である旨のメッセージを表示する
            this.LabelWaitMsg.Show();
            this.LabelWaitMsg.Location = new System.Drawing.Point(0, 0);
            this.LabelWaitMsg.Size = new Size(this.Width, this.Height);
        }

        /// <summary>
        /// Excel出力中である旨のメッセージを非表示にする
        /// </summary>
        private void HideWaitMsg()
        {
            //このフォームを操作可にする
            this.Enabled = true;

            //処理中のメッセージを非表示にする
            this.LabelWaitMsg.Hide();

            //フォームを再表示する
            this.Show();

            //一番手前にくるように、最前面にセットしてすぐ戻す
            this.TopMost = true;
            this.TopMost = false;
        }

        /// <summary>
        /// 終了ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
